@@ -834,6 +834,8 @@ reinit:
             now = time(NULL);
             if (frrequested &&
                 (now - frrequested) < params.iprop_resync_timeout) {
+                if (debug)
+                    fprintf(stderr, _("Still waiting for full resync\n"));
                 break;
             } else {
                 frrequested = now;
@@ -1440,8 +1442,8 @@ recv_database(context, fd, database_fd, confmsg)
     int database_fd;
     krb5_data *confmsg;
 {
-    krb5_ui_4       database_size; /* This must be 4 bytes */
-    int     received_size, n;
+    krb5_ui_4       database_size, received_size;
+    int             n;
     char            buf[1024];
     krb5_data       inbuf, outbuf;
     krb5_error_code retval;
@@ -1521,7 +1523,7 @@ recv_database(context, fd, database_fd, confmsg)
                      "while writing database block starting at offset %d",
                      received_size);
             send_error(context, fd, errno, buf);
-        } else if (n != outbuf.length) {
+        } else if ((unsigned int)n != outbuf.length) {
             snprintf(buf, sizeof(buf),
                      "incomplete write while writing database block starting at \noffset %d (%d written, %d expected)",
                      received_size, n, outbuf.length);
